#include "pv34.h"

pv34::pv34(OBJHANDLE hObj, int fmodel):VESSEL3(hObj,fmodel)
{
	hMesh = oapiLoadMeshGlobal("S4_PV");
	//SetEmptyMass(1814);

	aSARJ = CreateAttachment(true,_V(-0.01,0.38,0.5),_V(0,0,1),_V(0,-1,0),"SARJ");
	aT5 = CreateAttachment(false,_V(-0.01,0.38,-5.33),_V(0,1,0),_V(0,0,1),"T5");

	bottomAssembly.Set(AnimState::CLOSED,0.0);
	topAssembly.Set(AnimState::CLOSED,0.0);
	topArrayRot = bottomArrayRot = 0;

	topTracking = bottomTracking = false;
	topArrayTgt = bottomArrayTgt = 0;
	arrayRotRate = 1.0;

	radiator.Set(AnimState::CLOSED,0.0);

	
}

void pv34::DefineAnimations()
{


	//BOTTOM ASSEMBLY ROTATE UP
	animBottomAssembly = CreateAnimation(0);

	//comp 0
	static UINT bottomGrp1[1] = {47};
	rotBottomAssembly[0] = new MGROUP_ROTATE(uMeshIdx,bottomGrp1,1,
		_V(0.81,0.82,-5.33),_V(1,0,0),(float)-120/180*PI);
	bottomAssembly0 = AddAnimationComponent(animBottomAssembly,0,0.15,rotBottomAssembly[0]);

	//comp 1
	static UINT bottomGrp2[1] = {46};
	rotBottomAssembly[1] = new MGROUP_ROTATE(uMeshIdx,bottomGrp2,1,
		_V(0.80,0.30,-5.33),_V(1,0,0),(float)-120/180*PI);
	AddAnimationComponent(animBottomAssembly,0,0.15,rotBottomAssembly[1]);

	//comp 2
	static UINT bottomGrp3[9] = {30,31,39,40,41,42,43,44,45};
	rotBottomAssembly[2] = new MGROUP_ROTATE(uMeshIdx,bottomGrp3,9,
		_V(0.81,2.10,-5.73),_V(1,0,0),(float)120/180*PI);
	bottomAssembly2 = AddAnimationComponent(animBottomAssembly,0,0.15,rotBottomAssembly[2],bottomAssembly0);

	//comp3
	static UINT bottomGrp4[5] = {24,25,27,32,34};
	rotBottomAssembly[3] = new MGROUP_ROTATE(uMeshIdx,bottomGrp4,5,
		_V(1.39,-1.19,-5.82),_V(0,1,0),(float)PI/2);
	bottomAssembly3 = AddAnimationComponent(animBottomAssembly,0.3,0.35,rotBottomAssembly[3],bottomAssembly2);

	//comp4
	static UINT bottomGrp5[5] = {26,28,29,33,35};
	rotBottomAssembly[4] = new MGROUP_ROTATE(uMeshIdx,bottomGrp5,5,
		_V(0.22,-1.19,-5.83),_V(0,1,0),(float)-PI/2);
	bottomAssembly4 = AddAnimationComponent(animBottomAssembly,0.4,0.45,rotBottomAssembly[4],bottomAssembly2);

	//comp5
	static UINT bottomGrp6[1] = {36};
	scaleBottomAssembly[0] = new MGROUP_SCALE(uMeshIdx,bottomGrp6,1,_V(0.0,-1.00,-2.91),_V(1,136.0,1));
	AddAnimationComponent(animBottomAssembly,0.6,1,scaleBottomAssembly[0],bottomAssembly3);

	//comp6
	static UINT bottomGrp7[1] = {37};
	scaleBottomAssembly[1] = new MGROUP_SCALE(uMeshIdx,bottomGrp7,1,_V(0.0,-1.00,-2.91),_V(1,136.0,1));
	AddAnimationComponent(animBottomAssembly,0.6,1,scaleBottomAssembly[1],bottomAssembly4);

	//comp7
	static UINT bottomGrp8[1] = {38};
	scaleBottomAssembly[2] = new MGROUP_SCALE(uMeshIdx,bottomGrp8,1,_V(0.80,-0.26,-6.09),_V(1,39.2,1));
	AddAnimationComponent(animBottomAssembly,0.6,1,scaleBottomAssembly[2],bottomAssembly2);

	//comp8
	static UINT bottomGrp9[8] = {24,25,26,27,28,29,30,31};
	transBottomAssembly = new MGROUP_TRANSLATE(uMeshIdx,bottomGrp9,8,_V(0,-34.5,0));
	AddAnimationComponent(animBottomAssembly,0.6,1,transBottomAssembly);


	//BOTTOM ARRAY ROTATION
	animBottomArray = CreateAnimation(0);
	static UINT bottomArrayGrp[19] = {24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,43,40,44,45};
	rotBottomArray = new MGROUP_ROTATE(uMeshIdx,bottomArrayGrp,19,
		_V(0.80,-0.29,-6.51),_V(0,1,0),(float)2*PI);
	AddAnimationComponent(animBottomArray,0,1,rotBottomArray);



	//TOP ASSEMBLY ROTATE UP
	//comp 10
	animTopAssembly = CreateAnimation(0);

	static UINT topGrp10[1] = {23};
	rotTopAssembly[0] = new MGROUP_ROTATE(uMeshIdx,topGrp10,1,
		_V(0.80,-0.08,-5.34),_V(1,0,0),(float)120/180*PI);
	topAssembly10 = AddAnimationComponent(animTopAssembly,0,0.15,rotTopAssembly[0]);

	//comp11
	static UINT topGrp11[1] = {22};
	rotTopAssembly[1] = new MGROUP_ROTATE(uMeshIdx,topGrp11,1,
		_V(0.79,0.45,-5.33),_V(1,0,0),(float)120/180*PI);
	AddAnimationComponent(animTopAssembly,0,0.15,rotTopAssembly[1]);

	//comp12
	static UINT topGrp12[9] = {0,3,15,16,17,18,19,20,21};
	rotTopAssembly[2] = new MGROUP_ROTATE(uMeshIdx,topGrp12,9,
		_V(0.80,-1.35,-5.73),_V(1,0,0),(float)-120/180*PI);
	topAssembly12 = AddAnimationComponent(animTopAssembly,0,0.15,rotTopAssembly[2],topAssembly10);

	//comp13
	static UINT topGrp13[5] = {1,2,4,9,10};
	rotTopAssembly[3] = new MGROUP_ROTATE(uMeshIdx,topGrp13,5,
		_V(-1.38,1.95,-5.82),_V(0,1,0),(float)-90/180*PI);
	topAssembly13 = AddAnimationComponent(animTopAssembly,0.3,0.35,rotTopAssembly[3],topAssembly12);

	//comp14
	static UINT topGrp14[5] = {5,6,7,8,11};
	rotTopAssembly[4] = new MGROUP_ROTATE(uMeshIdx,topGrp14,5,
		_V(-0.21,1.95,-5.83),_V(0,1,0),(float)90/180*PI);
	topAssembly14 = AddAnimationComponent(animTopAssembly,0.4,0.45,rotTopAssembly[4],topAssembly12);

	//comp15
	static UINT topGrp15[1] = {12};
	scaleTopAssembly[0] = new MGROUP_SCALE(uMeshIdx,topGrp15,1,
		_V(0.0,1.692,-2.88),_V(1,136.0,1));
	AddAnimationComponent(animTopAssembly,0.6,1,scaleTopAssembly[0],topAssembly13);

	//comp16
	static UINT topGrp16[1] = {13};
	scaleTopAssembly[1] = new MGROUP_SCALE(uMeshIdx,topGrp16,1,
		_V(0.0,1.692,-2.88),_V(1,136.0,1));
	AddAnimationComponent(animTopAssembly,0.6,1,scaleTopAssembly[1],topAssembly14);

	//comp17
	static UINT topGrp17[1] = {14};
	scaleTopAssembly[2] = new MGROUP_SCALE(uMeshIdx,topGrp17,1,
		_V(-0.79,1.02,-6.12),_V(1,39.2,1));
	AddAnimationComponent(animTopAssembly,0.6,1,scaleTopAssembly[2],topAssembly12);

	//comp18
	static UINT topGrp18[8] = {0,1,2,3,4,5,6,7};
	transTopAssembly = new MGROUP_TRANSLATE(uMeshIdx,topGrp18,8,_V(0,34.5,0));
	AddAnimationComponent(animTopAssembly,0.6,1,transTopAssembly);



	//TOP ARRAY ROTATION
	animTopArray = CreateAnimation(0);

	//comp19
	static UINT topArrayGrp[19] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,16,19,20,21};
	rotTopArray = new MGROUP_ROTATE(uMeshIdx,topArrayGrp,19,
		_V(-0.79,1.10,-6.52),_V(0,1,0),(float)2*PI);
	AddAnimationComponent(animTopArray,0,1,rotTopArray);






	//RADIATOR DEPLOY/STOW
	animRadiator = CreateAnimation(0);

	//comp20
	static UINT radGrp20[1] = {65};
	rotRad[0] = new MGROUP_ROTATE(uMeshIdx,radGrp20,1,
		_V(-1.46,0.55,-0.82),_V(0,0,1),(float)PI/2);
	radiator20 = AddAnimationComponent(animRadiator,0,1,rotRad[0]);

	//comp21
	static UINT radGrp21[1] = {64};
	rotRad[1] = new MGROUP_ROTATE(uMeshIdx,radGrp21,1,
		_V(-1.53,1.25,-0.82),_V(0,0,1),(float)-160/180*PI);
	radiator21 = AddAnimationComponent(animRadiator,0,1,rotRad[1],radiator20);

	//comp22
	static UINT radGrp22[1] = {63};
	rotRad[2] = new MGROUP_ROTATE(uMeshIdx,radGrp22,1,
		_V(-1.60,-0.54,-0.82),_V(0,0,1),(float)140/180*PI);
	radiator22 = AddAnimationComponent(animRadiator,0,1,rotRad[2],radiator21);

	//comp23
	static UINT radGrp23[1] = {62};
	rotRad[3] = new MGROUP_ROTATE(uMeshIdx,radGrp23,1,
		_V(-1.67,1.25,-0.82),_V(0,0,1),(float)-140/180*PI);
	radiator23 = AddAnimationComponent(animRadiator,0,1,rotRad[3],radiator22);

	//comp24
	static UINT radGrp24[1] = {61};
	rotRad[4] = new MGROUP_ROTATE(uMeshIdx,radGrp24,1,
		_V(-1.74,-0.54,-0.82),_V(0,0,1),(float)140/180*PI);
	radiator24 = AddAnimationComponent(animRadiator,0,1,rotRad[4],radiator23);

	//comp25
	static UINT radGrp25[1] = {60};
	rotRad[5] = new MGROUP_ROTATE (uMeshIdx,radGrp25,1,
		_V(-1.81,1.25,-0.82),_V(0,0,1),(float)-140/180*PI);
	radiator25 = AddAnimationComponent(animRadiator,0,1,rotRad[5],radiator24);

	//comp26
	static UINT radGrp26[1] = {59};
	rotRad[6] = new MGROUP_ROTATE(uMeshIdx,radGrp26,1,
		_V(-1.88,-0.54,-0.82),_V(0,0,1),(float)140/180*PI);
	radiator26 = AddAnimationComponent(animRadiator,0,1,rotRad[6],radiator25);

	//comp27
	static UINT radGrp27[2] = {57,58};
	rotRad[7] = new MGROUP_ROTATE(uMeshIdx,radGrp27,2,
		_V(-1.95,1.25,-0.82),_V(0,0,1),(float)-140/180*PI);
	AddAnimationComponent(animRadiator,0,1,rotRad[7],radiator26);

	//comp28
	static UINT radGrp28[1] = {73};
	rotRad[8] = new MGROUP_ROTATE(uMeshIdx,radGrp28,1,
		_V(-1.46,0.17,-0.80),_V(0,0,1),(float)-65/180*PI);
	radiator28 = AddAnimationComponent(animRadiator,0,1,rotRad[8]);

	//comp29
	static UINT radGrp29[1] = {72};
	rotRad[9] = new MGROUP_ROTATE(uMeshIdx,radGrp29,1,
		_V(-1.53,-0.54,-0.80),_V(0,0,1),(float)135/180*PI);
	radiator29 = AddAnimationComponent(animRadiator,0,1,rotRad[9],radiator28);

	//comp30
	static UINT radGrp30[1] = {71};
	rotRad[10] = new MGROUP_ROTATE(uMeshIdx,radGrp30,1,
		_V(-1.60,1.25,-0.80),_V(0,0,1),(float)-140/180*PI);
	radiator30 = AddAnimationComponent(animRadiator,0,1,rotRad[10],radiator29);

	//comp31
	static UINT radGrp31[1] = {70};
	rotRad[11] = new MGROUP_ROTATE(uMeshIdx,radGrp31,1,
		_V(-1.67,-0.54,-0.80),_V(0,0,1),(float)140/180*PI);
	radiator31 = AddAnimationComponent(animRadiator,0,1,rotRad[11],radiator30);

	//comp32
	static UINT radGrp32[1] = {69};
	rotRad[12] = new MGROUP_ROTATE(uMeshIdx,radGrp32,1,
		_V(-1.74,1.25,-0.80),_V(0,0,1),(float)-140/180*PI);
	radiator32 = AddAnimationComponent(animRadiator,0,1,rotRad[12],radiator31);

	//comp33
	static UINT radGrp33[1] = {68};
	rotRad[13] = new MGROUP_ROTATE(uMeshIdx,radGrp33,1,
		_V(-1.81,-0.54,-0.80),_V(0,0,1),(float)140/180*PI);
	radiator33 = AddAnimationComponent(animRadiator,0,1,rotRad[13],radiator32);

	//comp34
	static UINT radGrp34[1] = {67};
	rotRad[14] = new MGROUP_ROTATE(uMeshIdx,radGrp34,1,
		_V(-1.88,1.25,-0.80),_V(0,0,1),(float)-140/180*PI);
	radiator34 = AddAnimationComponent(animRadiator,0,1,rotRad[14],radiator33);

	//comp35 finally, YEAH!!!
	static UINT radGrp35[1] = {66};
	rotRad[15] = new MGROUP_ROTATE(uMeshIdx,radGrp35,1,
		_V(-1.95,-0.54,-0.80),_V(0,0,1),(float)140/180*PI);
	AddAnimationComponent(animRadiator,0,1,rotRad[15],radiator34);

}

bool pv34::clbkDrawHUD(int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	double w = hps->W;
	DWORD csize = skp->GetCharSize();


	//CAPTIONS
	skp->SetTextColor(RGB(255,255,255));

	char cRadiator[] = "Radiator status";
	skp->Text(20,180,cRadiator,strlen(cRadiator));

	char cBottomAssembly[] = "Bottom assembly status:";
	skp->Text(20,200,cBottomAssembly,strlen(cBottomAssembly));

	char cTopAssembly[] = "Top assembly status:";
	skp->Text(20,220,cTopAssembly,strlen(cTopAssembly));

	char cTopArrayTrack[] = "Top array sun tracking:";
	skp->Text(20,240,cTopArrayTrack,strlen(cTopArrayTrack));

	char cBottomArrayTrack[] = "Bottom array sun tracking:";
	skp->Text(20,260,cBottomArrayTrack,strlen(cBottomArrayTrack));

	char cTopArrayAngle[] = "Top array rotation:";
	skp->Text(20,280,cTopArrayAngle,strlen(cTopArrayAngle));

	char cBottomArrayAngle[] = "Bottom array rotation:";
	skp->Text(20,300,cBottomArrayAngle,strlen(cBottomArrayAngle));

	char cTopArrayTgt[] = "Top array target angle:";
	skp->Text(20,320,cTopArrayTgt,strlen(cTopArrayTgt));

	char cBottomArrayTgt[] = "Bottom array target angle:";
	skp->Text(20,340,cBottomArrayTgt,strlen(cBottomArrayTgt));

	char cRotRate[] = "Array rotation rate:";
	skp->Text(20,360,cRotRate,strlen(cRotRate));



	char cbuf[256];


	//TOP ASSEMBLY STATES
	if(topAssembly.Closed())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Closed");
	}
	else if(topAssembly.Open())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Opened");
	}
	else if(topAssembly.Opening())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Opening");
	}
	else if(topAssembly.Closing())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Closing");
	}
	else if(topAssembly.Stopped())
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Stopped");
	}

	skp->Text(300,220,cbuf,strlen(cbuf));


	//BOTTOM ASSEMBLY STATES
	if(bottomAssembly.Closed())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Closed");
	}
	else if(bottomAssembly.Open())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Opened");
	}
	else if(bottomAssembly.Opening())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Opening");
	}
	else if(bottomAssembly.Closing())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Closing");
	}
	else if(bottomAssembly.Stopped())
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Stopped");
	}
	
	skp->Text(300,200,cbuf,strlen(cbuf));


	//TRACKING STATES
	if(topTracking)
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Enabled");
	}
	else
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Disabled");
	}
	skp->Text(300,240,cbuf,strlen(cbuf));

	if(bottomTracking)
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Enabled");
	}
	else
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Disabled");
	}
	skp->Text(300,260,cbuf,strlen(cbuf));

	//RADIATOR STATES
	if(radiator.Closed())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Closed");
	}
	else if(radiator.Open())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Opened");
	}
	else if(radiator.Opening())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Opening");
	}
	else if(radiator.Closing())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Closing");
	}
	else if(radiator.Stopped())
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Stopped");
	}
	
	skp->Text(300,180,cbuf,strlen(cbuf));
	

	//ARRAY ANGLES
	skp->SetTextColor(RGB(255,255,0));
	sprintf(cbuf,"%.2lf",topArrayRot*360);
	skp->Text(300,280,cbuf,strlen(cbuf));

	sprintf(cbuf,"%.2lf",bottomArrayRot*360);
	skp->Text(300,300,cbuf,strlen(cbuf));


	//TARGET ANGLES
	skp->SetTextColor(RGB(255,125,10));
	sprintf(cbuf,"%.2lf",topArrayTgt*360);
	skp->Text(300,320,cbuf,strlen(cbuf));

	sprintf(cbuf,"%.2lf",bottomArrayTgt*360);
	skp->Text(300,340,cbuf,strlen(cbuf));


	//ROTATION RATE
	skp->SetTextColor(RGB(177,56,48));
	sprintf(cbuf,"%.4lf",arrayRotRate);
	skp->Text(300,360,cbuf,strlen(cbuf));


	return true;

}

void pv34::clbkSetClassCaps(FILEHANDLE cfg)
{
	uMeshIdx = AddMesh(hMesh);
	DefineAnimations();
}

void pv34::clbkPostStep(double simt, double simdt, double mjd)
{
	if(bottomAssembly.Moving())
	{
		bottomAssembly.Move(0.001*simdt);
		SetAnimation(animBottomAssembly,bottomAssembly.pos);
	}

	if(topAssembly.Moving())
	{
		topAssembly.Move(0.001*simdt);
		SetAnimation(animTopAssembly,topAssembly.pos);
	}

	if(radiator.Moving())
	{
		radiator.Move(0.001*simdt);
		SetAnimation(animRadiator,radiator.pos);
	}

	if(topTracking)
	{
		if(topAssembly.Open())
		{

			topArrayTgt = CalculateSunTrackingAngle()/(PI*2);
			double d = topArrayTgt - topArrayRot;

			if(d>0)
			{
				if(fabs(d) >= arrayRotRate/360)
					topArrayRot += (arrayRotRate/360)*simdt;
				else 
					topArrayRot += fabs(d)*simdt;
			}
			else
			{
				if(fabs(d) >= arrayRotRate/360)
					topArrayRot -= (arrayRotRate/360)*simdt;
				else 
					topArrayRot -= fabs(d)*simdt;
			}

			if(topArrayRot > 1)
				topArrayRot = 0;

			if(topArrayRot < 0)
				topArrayRot = 1;


			SetAnimation(animTopArray,topArrayRot);
		}
	}

	else //navigate by custom target
	{
		if(topAssembly.Open())
		{

			double d = topArrayTgt - topArrayRot;

			if(d>0)
			{
				if(fabs(d) >= arrayRotRate/360)
					topArrayRot += (arrayRotRate/360)*simdt;
				else 
					topArrayRot += fabs(d)*simdt;
			}
			else
			{
				if(fabs(d) >= arrayRotRate/360)
					topArrayRot -= (arrayRotRate/360)*simdt;
				else 
					topArrayRot -= fabs(d)*simdt;
			}

			if(topArrayRot > 1)
				topArrayRot = 0;

			if(topArrayRot < 0)
				topArrayRot = 1;


			SetAnimation(animTopArray,topArrayRot);
		}
	} //topTracking



	if(bottomTracking)
	{
		if(bottomAssembly.Open())
		{

			bottomArrayTgt = CalculateSunTrackingAngle()/(PI*2);
			double d = bottomArrayTgt - bottomArrayRot;

			if(d>0)
			{
				if(fabs(d) >= arrayRotRate/360)
					bottomArrayRot += (arrayRotRate/360)*simdt;
				else 
					bottomArrayRot += fabs(d)*simdt;
			}
			else
			{
				if(fabs(d) >= arrayRotRate/360)
					bottomArrayRot -= (arrayRotRate/360)*simdt;
				else 
					bottomArrayRot -= fabs(d)*simdt;
			}

			if(bottomArrayRot > 1)
				bottomArrayRot = 0;

			if(bottomArrayRot < 0)
				bottomArrayRot = 1;


			SetAnimation(animBottomArray,bottomArrayRot);
		}
	}
	else
	{
		if(bottomAssembly.Open())
		{

			double d = bottomArrayTgt - bottomArrayRot;

			if(d>0)
			{
				if(fabs(d) >= arrayRotRate/360)
					bottomArrayRot += (arrayRotRate/360)*simdt;
				else 
					bottomArrayRot += fabs(d)*simdt;
			}
			else
			{
				if(fabs(d) >= arrayRotRate/360)
					bottomArrayRot -= (arrayRotRate/360)*simdt;
				else 
					bottomArrayRot -= fabs(d)*simdt;
			}

			if(bottomArrayRot > 1)
				bottomArrayRot = 0;

			if(bottomArrayRot < 0)
				bottomArrayRot = 1;


			SetAnimation(animBottomArray,bottomArrayRot);
		}
	}//bottomTracking
	
}

void pv34::OperateBottomAssembly(bool open)
{
	if(open)
	{
		if(bottomAssembly.action != AnimState::OPEN)
			bottomAssembly.action = AnimState::OPENING;
	}
	else
	{
		if(bottomArrayRot == 0 && !bottomTracking)
		{
			if(bottomAssembly.action != AnimState::CLOSED)
				bottomAssembly.action = AnimState::CLOSING;
		}
		else
		{
			sprintf(oapiDebugString(),"Rotate bottom array to 0 angle first");
		}
	}
}

void pv34::OperateTopAssembly(bool open)
{
	AnimState::Action action = topAssembly.action;

	if(open)
	{
		if(action != AnimState::OPEN)
			topAssembly.action = AnimState::OPENING;
	}
	else
	{
		if(topArrayRot == 0 && !topTracking)
		{
			if(action != AnimState::CLOSED)
				topAssembly.action = AnimState::CLOSING;
		}
		else
		{
			sprintf(oapiDebugString(),"Rotate top array to 0 angle first");
		}
	}
}

void pv34::OperateRadiator(bool open)
{
	AnimState::Action action = radiator.action;

	if(open)
	{
		if(action != AnimState::OPEN)
			radiator.action = AnimState::OPENING;
	}
	else
	{
		if(action != AnimState::CLOSED)
			radiator.action = AnimState::CLOSING;
	}
}

int pv34::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	if(!down)
		return 0;

	if(KEYMOD_CONTROL(kstate))
	{
		switch(key)
		{
		case OAPI_KEY_1: {oapiOpenInputBox("Top array target angle[0-359]",clbkTopArrayTgt,"",20,this); return 1;}
		case OAPI_KEY_2: {oapiOpenInputBox("Bottom array target angle[0-359]",clbkBottomArrayTgt,"",20,this); return 1;}
		case OAPI_KEY_3: {oapiOpenInputBox("Array rotation rate[>=0]",clbkRotRate,"",20,this); return 1;}
		default: return 0;
		}
	}

	switch(key)
	{
	case OAPI_KEY_NUMPAD7: {OperateBottomAssembly(true); return 1;}
	case OAPI_KEY_NUMPAD4: {bottomAssembly.action = AnimState::STOPPED; return 1;}
	case OAPI_KEY_NUMPAD1: {OperateBottomAssembly(false); return 1;}
	case OAPI_KEY_NUMPAD8: {OperateTopAssembly(true); return 1;}
	case OAPI_KEY_NUMPAD5: {topAssembly.action = AnimState::STOPPED; return 1;}
	case OAPI_KEY_NUMPAD2: {OperateTopAssembly(false); return 1;}
	case OAPI_KEY_NUMPAD9: {OperateRadiator(true); return 1;}
	case OAPI_KEY_NUMPAD6: {radiator.action = AnimState::STOPPED; return 1;}
	case OAPI_KEY_NUMPAD3: {OperateRadiator(false); return 1;}
	case OAPI_KEY_1: {if(topTracking) topTracking = false; else topTracking = true; return 1;}
	case OAPI_KEY_2: {if(bottomTracking) bottomTracking = false; else bottomTracking = true; return 1;}
	default: return 0;
	}
}

bool pv34::clbkBottomArrayTgt(void *id, char *str, void *usrdata)
{
	pv34* pv = (pv34*)usrdata;

	double t;
	sscanf(str,"%lf",&t);

	if(t>=0 && t<=359)
	{
		pv->bottomArrayTgt = t/360;
		return true;
	}
	else return false;
}

bool pv34::clbkTopArrayTgt(void *id, char *str, void *usrdata)
{
	pv34* pv = (pv34*)usrdata;

	double t;
	sscanf(str,"%lf",&t);

	if(t>=0 && t<=359)
	{
		pv->topArrayTgt = t/360;
		return true;
	}
	else return false;
}

bool pv34::clbkRotRate(void *id, char *str, void *usrdata)
{
	pv34 *pv = (pv34*)usrdata;

	double t;
	sscanf(str,"%lf",&t);
	if(t>=0)
	{
		pv->arrayRotRate = t;
		return true;
	}
	else return false;
}

double pv34::CalculateSunTrackingAngle()
{
	VECTOR3 toSun, SunPos, ShipPos;
	oapiGetGlobalPos(oapiGetGbodyByName("Sun"), &SunPos);
	GetGlobalPos(ShipPos);

	MATRIX3 R;
	GetRotationMatrix(R);
	toSun = tmul(R, SunPos - ShipPos);

	double ang = atan2(toSun.x, toSun.z);
	if(ang < 0) ang += PI*2;

	return ang;
}

void pv34::clbkSaveState(FILEHANDLE scn)
{
	VESSEL3::clbkSaveState(scn);

	char cbuf[255];

	sprintf(cbuf,"%lf",arrayRotRate);
	oapiWriteScenario_string(scn,"ARRAY_RATE",cbuf);

	oapiWriteScenario_int(scn,"TOP_TRACKING",topTracking);
	oapiWriteScenario_int(scn,"BOTTOM_TRACKING",bottomTracking);

	sprintf(cbuf,"%lf",topArrayTgt);
	oapiWriteScenario_string(scn,"TOP_TGT",cbuf);

	sprintf(cbuf,"%lf",bottomArrayTgt);
	oapiWriteScenario_string(scn,"BOTTOM_TGT",cbuf);

	sprintf(cbuf,"%lf",topArrayRot);
	oapiWriteScenario_string(scn,"TOP_ROT",cbuf);

	sprintf(cbuf,"%lf",bottomArrayRot);
	oapiWriteScenario_string(scn,"BOTTOM_ROT",cbuf);

	sprintf(cbuf,"%i %lf",bottomAssembly.action,bottomAssembly.pos);
	oapiWriteScenario_string(scn,"ANIM_BOTTOM",cbuf);

	sprintf(cbuf,"%i %lf",topAssembly.action,topAssembly.pos);
	oapiWriteScenario_string(scn,"ANIM_TOP",cbuf);
	
	sprintf(cbuf,"%i %lf",radiator.action,radiator.pos);
	oapiWriteScenario_string(scn,"ANIM_RAD",cbuf);
	
}

void pv34::clbkLoadStateEx(FILEHANDLE scn, void *status)
{
	char *line;

	while(oapiReadScenario_nextline(scn,line))
	{
		if(!strnicmp(line,"ARRAY_RATE",10))
		{
			sscanf(line+10,"%lf",&arrayRotRate);
		}
		else if(!strnicmp(line,"TOP_TRACKING",12))
		{
			sscanf(line+12,"%i",&topTracking);
		}
		else if(!strnicmp(line,"BOTTOM_TRACKING",15))
		{
			sscanf(line+15,"%i",&bottomTracking);
		}
		else if(!strnicmp(line,"TOP_TGT",7))
		{
			sscanf(line+7,"%lf",&topArrayTgt);
		}
		else if(!strnicmp(line,"BOTTOM_TGT",10))
		{
			sscanf(line+10,"%lf",&bottomArrayTgt);
		}
		else if(!strnicmp(line,"TOP_ROT",7))
		{
			sscanf(line+7,"%lf",&topArrayRot);
			SetAnimation(animTopArray,topArrayRot/360);
		}
		else if(!strnicmp(line,"BOTTOM_ROT",10))
		{
			sscanf(line+10,"%lf",&bottomArrayRot);
			SetAnimation(animBottomArray,bottomArrayRot/360);
		}
		else if(!strnicmp(line,"ANIM_BOTTOM",11))
		{
			int state; 
			double pos;
			sscanf(line+11,"%i %lf",&state,&pos);
			bottomAssembly.Set(GetAction(state),pos);
			SetAnimation(animBottomAssembly,pos);
		}
		else if(!strnicmp(line,"ANIM_TOP",8))
		{
			int state;
			double pos;
			sscanf(line+8,"%i %lf",&state,&pos);
			topAssembly.Set(GetAction(state),pos);
			SetAnimation(animTopAssembly,pos);
		}
		else if(!strnicmp(line,"ANIM_RAD",8))
		{
			int state;
			double pos;
			sscanf(line+8,"%i %lf",&state,&pos);
			radiator.Set(GetAction(state),pos);
			SetAnimation(animRadiator,pos);
		}
		else
			ParseScenarioLineEx(line,status);
	}

}

AnimState::Action pv34::GetAction(int a)
{
	switch(a)
	{
	case 1: return AnimState::CLOSED;
	case 2: return AnimState::OPEN;
	case 3: return AnimState::CLOSING;
	case 4: return AnimState::OPENING;
	case 0: return AnimState::STOPPED;
	}
}

DLLCLBK void InitModule (HINSTANCE hModule)
{
	g_hDLL = hModule;
	// perform global module initialisation here
}

DLLCLBK void ExitModule (HINSTANCE hModule)
{
	// perform module cleanup here
}

DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel) 
{ 
	return new pv34(hvessel, flightmodel); 
}

DLLCLBK void ovcExit (VESSEL *vessel) 
{ 
	if (vessel) 
		delete (pv34*)vessel; 
}

pv34::~pv34()
{

	for(int i=0; i<5; i++)
	{
		delete rotBottomAssembly[i];
		delete rotTopAssembly[i];
	}

	for(int i=0; i<3; i++)
	{
		delete scaleBottomAssembly[i];
		delete scaleTopAssembly[i];
	}

	delete transBottomAssembly;
	delete transTopAssembly;
	delete rotBottomArray;
	delete rotTopArray;
	
	for(int i=0; i<16; i++)
	{
		delete rotRad[i];
	}


}
