#include "pv34_sarj.h"


pv34_sarj::pv34_sarj(OBJHANDLE hObj, int fmodel):VESSEL3(hObj,fmodel)
{
	//hMesh = oapiLoadMeshGlobal("P3_SARJ");

	//hDock = CreateDock(_V(-0.01,0.38,6.06),_V(0,-1,0),_V(0,0,1));

	//attP4 = CreateAttachment(false,_V(-0.01,0.38,0.5),_V(0,0,-1),_V(0,1,0),"P4");
	attP4 = CreateAttachment(false,_V(0,0,0),_V(0,0,-1),_V(0,1,0),"P4");
	attP3S3 = CreateAttachment(true,_V(0,0,0),_V(0,0,1),_V(0,1,0),"P3");
	//attESP1 = CreateAttachment(false,_V(-2.08,1.09,2.10),_V(-1,0,0),_V(0,0,-1),"XS");
	//attESP2 = CreateAttachment(false,_V(2.06,1.09,2.10),_V(-1,0,0),_V(0,0,1),"XS");
	//attBay = CreateAttachment(true,_V(0,0,0),_V(0,1,0),_V(0,0,1),"XS");
	//attPORT = CreateAttachment(true,_V(-1.22,1.72,3.37),_V(-0.755069,0.655639,0.00280868),_V(-0.611114,-0.791271,-0.02073),"GS");
	//attSTBD = CreateAttachment(true,_V(1.20,1.72,3.37),_V(0.755069,-0.655639,-0.00280868),_V(-0.611114,-0.791271,-0.02073),"GS");

	tgtAngle = currAngle = 0;
	rotSpeed = PI/180; //1 degree per second
	tracking = false;
	bOpp = false;

}


pv34_sarj::~pv34_sarj()
{


}

void pv34_sarj::clbkSetClassCaps(FILEHANDLE cfg)
{
	//AddMesh(hMesh);
	SetEmptyMass(1814);
}

void pv34_sarj::clbkPostStep(double simt, double simdt, double mjd)
{	
	VECTOR3 toSun, SunPos, ShipPos;
	oapiGetGlobalPos(oapiGetGbodyByName("Sun"), &SunPos);
	GetGlobalPos(ShipPos);

	MATRIX3 R;
	GetRotationMatrix(R);
	toSun = tmul(R, SunPos - ShipPos);

	double ang = atan2(toSun.x, toSun.y);
	if(ang < 0) ang += PI*2;

	if(bOpp)
	{
		if(ang > PI)
			ang -= PI;
		else ang += PI;
	}


	if(tracking)
	{
		tgtAngle = ang;
		double diff = tgtAngle - currAngle;
		
		if(fabs(diff) <= rotSpeed) //difference same or less than rotSpeed
		{
				currAngle += diff*simdt;
		}
		else
		{
			if(diff<0)
				currAngle -= rotSpeed*simdt;
			else if (diff > 0)
				currAngle += rotSpeed*simdt;
		}

		if(currAngle > 2*PI)
			currAngle = 0;

		if(currAngle < 0)
			currAngle = 2*PI;	

		
	}
	else
	{
		double diff = tgtAngle - currAngle;
		
		if(fabs(diff) <= rotSpeed) //difference same or less than rotSpeed
		{
			currAngle += diff*simdt;
		}
		else
		{
			if(diff<0)
				currAngle -= rotSpeed*simdt;
			else
				currAngle += rotSpeed*simdt;
		}

		if(currAngle > 2*PI)
			currAngle = 0;

		if(currAngle < 0)
			currAngle = 2*PI;

		
	}

	if(tgtAngle != currAngle)
	{
		RotateAttachment(currAngle);
	}


}

void pv34_sarj::RotateAttachment(double angle)
{
	VECTOR3 dir,pos,rot;
	GetAttachmentParams(attP4,pos,dir,rot);

	Quaternion q = Quaternion(_V(0,0,1),angle+PI/2);
	VECTOR3 new_rot = q.transform(_V(0,1,0));
	normalise(new_rot);

	SetAttachmentParams(attP4,pos,dir,new_rot);
}

int pv34_sarj::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	if(!down)
		return 0;

	switch(key)
	{
	case OAPI_KEY_1: {if(!tracking) tracking = true; else tracking = false; return 1;}
	case OAPI_KEY_2: {if(!bOpp) bOpp = true; else bOpp = false; return 1;}
	case OAPI_KEY_3: {oapiOpenInputBox("Enter target angle[0-360]",clbkTgtAngle,"",20,this); return 1;}
	case OAPI_KEY_4: {oapiOpenInputBox("Enter rotation rate[>=0]",clbkTgtRate,"",20,this); return 1;}
	default: return 0;
	}
}

bool pv34_sarj::clbkTgtAngle(void *id, char *str, void *usrdata)
{
	pv34_sarj *pv = (pv34_sarj*)usrdata;

	double t;
	sscanf(str,"%lf",&t);
	if(t<0 || t>360)
		return false;
	else
	{
		pv->tgtAngle = t/180*PI;
		return true;
	}
}

bool pv34_sarj::clbkTgtRate(void *id, char *str, void *usrdata)
{
	pv34_sarj *pv = (pv34_sarj*)usrdata;

	double t;
	sscanf(str,"%lf",&t);
	if(t>=0)
	{
		pv->rotSpeed = t/180*PI;
		return true;
	}
	else return false;
}

bool pv34_sarj::clbkDrawHUD(int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	double w = hps->W;

	//constant captions
	skp->SetTextColor(RGB(255,255,255));
	
	if(!skp->Text(20,200,"Sun tracking:",13))
		return false;

	if(!skp->Text(20,220,"Current angle:",14))
		return false;

	if(!skp->Text(20,240,"Target angle:",13))
		return false;

	if(!skp->Text(20,260,"Rotation speed",14))
		return false;

	if(!skp->Text(20,280,"Sun opposite",12))
		return false;

	//indicators
	skp->SetTextColor(RGB(0,255,0));

	if(tracking)
	{
		if(!skp->Text(200,200,"ENABLED",7))
			return false;
	}
	else
	{
		if(!skp->Text(200,200,"DISABLED",8))
			return false;
	}

	char cbuf[255];

	sprintf(cbuf,"%lf",currAngle/PI*180);
	if(!skp->Text(200,220,cbuf,strlen(cbuf)))
		return false;

	sprintf(cbuf,"%lf",tgtAngle/PI*180);
	if(!skp->Text(200,240,cbuf,strlen(cbuf)))
		return false;

	sprintf(cbuf,"%lf",rotSpeed/PI*180);
	if(!skp->Text(200,260,cbuf,strlen(cbuf)))
		return false;

	if(bOpp)
	{
		if(!skp->Text(200,280,"ENABLED",7))
			return false;
	}
	else
	{
		if(!skp->Text(200,280,"DISABLED",8))
			return false;
	}

	return true;
}

void pv34_sarj::clbkSaveState(FILEHANDLE scn)
{
	VESSEL3::clbkSaveState(scn);

	char cbuf[255];

	oapiWriteScenario_int(scn,"TRACKING",tracking);
	
	sprintf(cbuf,"%lf",currAngle);
	oapiWriteScenario_string(scn,"CURR_ANG",cbuf);

	sprintf(cbuf,"%lf",tgtAngle);
	oapiWriteScenario_string(scn,"TGT_ANG",cbuf);

	sprintf(cbuf,"%lf",rotSpeed);
	oapiWriteScenario_string(scn,"ROT_SPEED",cbuf);
}

void pv34_sarj::clbkLoadStateEx(FILEHANDLE scn, void *status)
{
	char *line;
	while(oapiReadScenario_nextline(scn,line))
	{
		if(!strnicmp(line,"TRACKING",8))
		{
			sscanf(line+8,"%i",&tracking);
		}
		else if(!strnicmp(line,"CURR_ANG",8))
		{
			sscanf(line+8,"%lf",&currAngle);
			RotateAttachment(currAngle);
		}
		else if(!strnicmp(line,"TGT_ANG",7))
		{
			sscanf(line+7,"%lf",&tgtAngle);
		}
		else if(!strnicmp(line,"ROT_SPEED",9))
		{
			sscanf(line+9,"%lf",&rotSpeed);
		}
		else
			ParseScenarioLineEx(line,status);
	}
}

DLLCLBK void InitModule (HINSTANCE hModule)
{
	g_hDLL = hModule;
	// perform global module initialisation here
}

DLLCLBK void ExitModule (HINSTANCE hModule)
{
	// perform module cleanup here
}

DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel) 
{ 
	return new pv34_sarj(hvessel, flightmodel); 
}

DLLCLBK void ovcExit (VESSEL *vessel) 
{ 
	if (vessel) 
		delete (pv34_sarj*)vessel; 
}

