#include "quaternion.h"
#include <math.h>

Quaternion::Quaternion()
{
	a = b = c = d = 0;
}

Quaternion::Quaternion(VECTOR3 axis, double ang)
{
	a = cos(ang/2);
	b = -sin(ang/2)*axis.x;
	c = sin(ang/2)*axis.y;
	d = sin(ang/2)*axis.z;
}

Quaternion::Quaternion(double x, double y, double z, double w)
{
	a = x;
	b = y;
	c = z;
	d = w;
}

Quaternion::~Quaternion()
{
}

Quaternion& Quaternion::operator=(const Quaternion& arg)
{
	a = arg.a;
	b = arg.b;
	c = arg.c;
	d = arg.d;
	return(*this);
}

Quaternion Quaternion::operator*(const Quaternion& arg)
{
	double ra, rb, rc, rd;
	ra = a*arg.a - b*arg.b - c*arg.c - d*arg.d;
	rb = a*arg.b + b*arg.a + c*arg.d - d*arg.c;
	rc = a*arg.c + c*arg.a + d*arg.b - b*arg.d;
	rd = a*arg.d + d*arg.a + b*arg.c - c*arg.b;
	return Quaternion(ra, rb, rc, rd);
}

Quaternion Quaternion::conj()
{
	return Quaternion(a, -b, -c, -d);
}

Quaternion Quaternion::inv()
{
	double l = a*a + b*b + c*c + d*d;
	return Quaternion(a/l, -b/l, -c/l, -d/l);
}

VECTOR3 Quaternion::rotationAxis()
{
	double l = sqrt(b*b + c*c + d*d);
	if(a<0) l = -l;
	if(l == 0.0) return _V(0.0, 0.0, 0.0);
	return _V(-b/l, c/l, d/l);
}

double Quaternion::rotationAngle()
{
	double l = sqrt(a*a + b*b + c*c + d*d);
	if(a<0) l = -l;
	double ang = 2*acos(a/l);
	if(ang > PI) return ang - 2*PI;
	return ang;
}

VECTOR3 Quaternion::transform(VECTOR3 v)
{
	Quaternion q(0.0, -v.x, v.y, v.z);
	Quaternion q2 = (*this) * q * this->inv();
	return _V(-q2.b, q2.c, q2.d);
}

Quaternion fromEuler(double p, double y, double r)
{
	Quaternion Qp(_V(1.0,0.0,0.0), p);
	Quaternion Qy(_V(0.0,-1.0,0.0), y);
	Quaternion Qr(_V(0.0,0.0,-1.0), r);
	return Qy*Qp*Qr;
}