#include "pv_p6.h"

pv_p6::pv_p6(OBJHANDLE hObj, int fmodel):VESSEL3(hObj,fmodel)
{
	hMesh = oapiLoadMeshGlobal("P6_PV");
	//SetEmptyMass(1814);

	aEAS = CreateAttachment(false,_V(2.566,0.748,4.238),_V(1,0,0),_V(0,0,1),"EAS");
	aPDGF = CreateAttachment(false, _V(-1.169,-0.992,3.487),_V(0.453987,0.891008,0),_V(0,0,1),"PDGF");
	aBay = CreateAttachment(true,_V(0,-2.2,-9.53),_V(0,-1,0),_V(0,0,1),"Bay");
	aGrTop = CreateAttachment(true,_V(-1.34,1.79,4.62),_V(-0.719205,0.643202,0.262746),_V(0.00332244,-0.377162,0.926142),"Top");
	aGrBottom = CreateAttachment(true,_V(1.58,-0.71,4.62),_V(0.865349,-0.000915766,0.50117),_V(-0.00318284,0.999938,0.0107054),"Bottom");
	aP5 = CreateAttachment(true,_V(0,0.39,6.97),_V(0,0,1),_V(0,1,0),"P5");
	aPDGFBottom = CreateAttachment(true,_V(-1.211,-1.084,3.486),_V(-0.454732,-0.890628,0),_V(0,0,1),"PDGF_B");
	aBottom2 = CreateAttachment(true,_V(1.02,-0.863,2.021),_V(0,-1,0),_V(0,0,1),"BOTTOM");

	bottomAssembly.Set(AnimState::CLOSED,0.0);
	topAssembly.Set(AnimState::CLOSED,0.0);
	topArrayRot = bottomArrayRot = 0;

	topTracking = bottomTracking = false;
	topArrayTgt = bottomArrayTgt = 0;
	arrayRotRate = 1.0;

	portRadiator.Set(AnimState::CLOSED,0.0);
	stbdRadiator.Set(AnimState::CLOSED,0.0);
	topRadiator.Set(AnimState::CLOSED,0.0);

	
}

void pv_p6::DefineAnimations()
{


	//BOTTOM ASSEMBLY ROTATE UP
	animBottomAssembly = CreateAnimation(0);

	//comp 0
	static UINT bottomGrp1[1] = {47};
	rotBottomAssembly[0] = new MGROUP_ROTATE(uMeshIdx,bottomGrp1,1,
		_V(0.81,0.82,-5.33),_V(1,0,0),(float)-120/180*PI);
	bottomAssembly0 = AddAnimationComponent(animBottomAssembly,0,0.15,rotBottomAssembly[0]);

	//comp 1
	static UINT bottomGrp2[1] = {46};
	rotBottomAssembly[1] = new MGROUP_ROTATE(uMeshIdx,bottomGrp2,1,
		_V(0.80,0.30,-5.33),_V(1,0,0),(float)-120/180*PI);
	AddAnimationComponent(animBottomAssembly,0,0.15,rotBottomAssembly[1]);

	//comp 2
	static UINT bottomGrp3[9] = {30,31,39,40,41,42,43,44,45};
	rotBottomAssembly[2] = new MGROUP_ROTATE(uMeshIdx,bottomGrp3,9,
		_V(0.81,2.10,-5.73),_V(1,0,0),(float)120/180*PI);
	bottomAssembly2 = AddAnimationComponent(animBottomAssembly,0,0.15,rotBottomAssembly[2],bottomAssembly0);

	//comp3
	static UINT bottomGrp4[5] = {24,25,27,32,34};
	rotBottomAssembly[3] = new MGROUP_ROTATE(uMeshIdx,bottomGrp4,5,
		_V(1.39,-1.19,-5.82),_V(0,1,0),(float)PI/2);
	bottomAssembly3 = AddAnimationComponent(animBottomAssembly,0.3,0.35,rotBottomAssembly[3],bottomAssembly2);

	//comp4
	static UINT bottomGrp5[5] = {26,28,29,33,35};
	rotBottomAssembly[4] = new MGROUP_ROTATE(uMeshIdx,bottomGrp5,5,
		_V(0.22,-1.19,-5.83),_V(0,1,0),(float)-PI/2);
	bottomAssembly4 = AddAnimationComponent(animBottomAssembly,0.4,0.45,rotBottomAssembly[4],bottomAssembly2);

	//comp5
	static UINT bottomGrp6[1] = {36};
	scaleBottomAssembly[0] = new MGROUP_SCALE(uMeshIdx,bottomGrp6,1,_V(0.0,-1.00,-2.91),_V(1,136.0,1));
	AddAnimationComponent(animBottomAssembly,0.6,1,scaleBottomAssembly[0],bottomAssembly3);

	//comp6
	static UINT bottomGrp7[1] = {37};
	scaleBottomAssembly[1] = new MGROUP_SCALE(uMeshIdx,bottomGrp7,1,_V(0.0,-1.00,-2.91),_V(1,136.0,1));
	AddAnimationComponent(animBottomAssembly,0.6,1,scaleBottomAssembly[1],bottomAssembly4);

	//comp7
	static UINT bottomGrp8[1] = {38};
	scaleBottomAssembly[2] = new MGROUP_SCALE(uMeshIdx,bottomGrp8,1,_V(0.80,-0.26,-6.09),_V(1,39.2,1));
	AddAnimationComponent(animBottomAssembly,0.6,1,scaleBottomAssembly[2],bottomAssembly2);

	//comp8
	static UINT bottomGrp9[8] = {24,25,26,27,28,29,30,31};
	transBottomAssembly = new MGROUP_TRANSLATE(uMeshIdx,bottomGrp9,8,_V(0,-34.5,0));
	AddAnimationComponent(animBottomAssembly,0.6,1,transBottomAssembly);


	//BOTTOM ARRAY ROTATION
	animBottomArray = CreateAnimation(0);
	static UINT bottomArrayGrp[19] = {24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,43,40,44,45};
	rotBottomArray = new MGROUP_ROTATE(uMeshIdx,bottomArrayGrp,19,
		_V(0.80,-0.29,-6.51),_V(0,1,0),(float)2*PI);
	AddAnimationComponent(animBottomArray,0,1,rotBottomArray);



	//TOP ASSEMBLY ROTATE UP
	//comp 10
	animTopAssembly = CreateAnimation(0);

	static UINT topGrp10[1] = {23};
	rotTopAssembly[0] = new MGROUP_ROTATE(uMeshIdx,topGrp10,1,
		_V(0.80,-0.08,-5.34),_V(1,0,0),(float)120/180*PI);
	topAssembly10 = AddAnimationComponent(animTopAssembly,0,0.15,rotTopAssembly[0]);

	//comp11
	static UINT topGrp11[1] = {22};
	rotTopAssembly[1] = new MGROUP_ROTATE(uMeshIdx,topGrp11,1,
		_V(0.79,0.45,-5.33),_V(1,0,0),(float)120/180*PI);
	AddAnimationComponent(animTopAssembly,0,0.15,rotTopAssembly[1]);

	//comp12
	static UINT topGrp12[9] = {0,3,15,16,17,18,19,20,21};
	rotTopAssembly[2] = new MGROUP_ROTATE(uMeshIdx,topGrp12,9,
		_V(0.80,-1.35,-5.73),_V(1,0,0),(float)-120/180*PI);
	topAssembly12 = AddAnimationComponent(animTopAssembly,0,0.15,rotTopAssembly[2],topAssembly10);

	//comp13
	static UINT topGrp13[5] = {1,2,4,9,10};
	rotTopAssembly[3] = new MGROUP_ROTATE(uMeshIdx,topGrp13,5,
		_V(-1.38,1.95,-5.82),_V(0,1,0),(float)-90/180*PI);
	topAssembly13 = AddAnimationComponent(animTopAssembly,0.3,0.35,rotTopAssembly[3],topAssembly12);

	//comp14
	static UINT topGrp14[5] = {5,6,7,8,11};
	rotTopAssembly[4] = new MGROUP_ROTATE(uMeshIdx,topGrp14,5,
		_V(-0.21,1.95,-5.83),_V(0,1,0),(float)90/180*PI);
	topAssembly14 = AddAnimationComponent(animTopAssembly,0.4,0.45,rotTopAssembly[4],topAssembly12);

	//comp15
	static UINT topGrp15[1] = {12};
	scaleTopAssembly[0] = new MGROUP_SCALE(uMeshIdx,topGrp15,1,
		_V(0.0,1.692,-2.88),_V(1,136.0,1));
	AddAnimationComponent(animTopAssembly,0.6,1,scaleTopAssembly[0],topAssembly13);

	//comp16
	static UINT topGrp16[1] = {13};
	scaleTopAssembly[1] = new MGROUP_SCALE(uMeshIdx,topGrp16,1,
		_V(0.0,1.692,-2.88),_V(1,136.0,1));
	AddAnimationComponent(animTopAssembly,0.6,1,scaleTopAssembly[1],topAssembly14);

	//comp17
	static UINT topGrp17[1] = {14};
	scaleTopAssembly[2] = new MGROUP_SCALE(uMeshIdx,topGrp17,1,
		_V(-0.79,1.02,-6.12),_V(1,39.2,1));
	AddAnimationComponent(animTopAssembly,0.6,1,scaleTopAssembly[2],topAssembly12);

	//comp18
	static UINT topGrp18[8] = {0,1,2,3,4,5,6,7};
	transTopAssembly = new MGROUP_TRANSLATE(uMeshIdx,topGrp18,8,_V(0,34.5,0));
	AddAnimationComponent(animTopAssembly,0.6,1,transTopAssembly);



	//TOP ARRAY ROTATION
	animTopArray = CreateAnimation(0);

	//comp19
	static UINT topArrayGrp[19] = {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,16,19,20,21};
	rotTopArray = new MGROUP_ROTATE(uMeshIdx,topArrayGrp,19,
		_V(-0.79,1.10,-6.52),_V(0,1,0),(float)2*PI);
	AddAnimationComponent(animTopArray,0,1,rotTopArray);






	//PORT RADIATOR DEPLOY/STOW
	animPortRadiator = CreateAnimation(0);

	//comp20
	static UINT radGrp20[1] = {91};
	portRotRad[0] = new MGROUP_ROTATE(uMeshIdx,radGrp20,1,
		_V(-1.46,0.55,-0.82),_V(0,0,1),(float)PI/2);
	radiator20 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[0]);

	//comp21
	static UINT radGrp21[1] = {90};
	portRotRad[1] = new MGROUP_ROTATE(uMeshIdx,radGrp21,1,
		_V(-1.53,1.25,-0.82),_V(0,0,1),(float)-160/180*PI);
	radiator21 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[1],radiator20);

	//comp22
	static UINT radGrp22[1] = {89};
	portRotRad[2] = new MGROUP_ROTATE(uMeshIdx,radGrp22,1,
		_V(-1.60,-0.54,-0.82),_V(0,0,1),(float)140/180*PI);
	radiator22 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[2],radiator21);

	//comp23
	static UINT radGrp23[1] = {88};
	portRotRad[3] = new MGROUP_ROTATE(uMeshIdx,radGrp23,1,
		_V(-1.67,1.25,-0.82),_V(0,0,1),(float)-140/180*PI);
	radiator23 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[3],radiator22);

	//comp24
	static UINT radGrp24[1] = {87};
	portRotRad[4] = new MGROUP_ROTATE(uMeshIdx,radGrp24,1,
		_V(-1.74,-0.54,-0.82),_V(0,0,1),(float)140/180*PI);
	radiator24 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[4],radiator23);

	//comp25
	static UINT radGrp25[1] = {86};
	portRotRad[5] = new MGROUP_ROTATE (uMeshIdx,radGrp25,1,
		_V(-1.81,1.25,-0.82),_V(0,0,1),(float)-140/180*PI);
	radiator25 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[5],radiator24);

	//comp26
	static UINT radGrp26[1] = {85};
	portRotRad[6] = new MGROUP_ROTATE(uMeshIdx,radGrp26,1,
		_V(-1.88,-0.54,-0.82),_V(0,0,1),(float)140/180*PI);
	radiator26 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[6],radiator25);

	//comp27
	static UINT radGrp27[2] = {83,84};
	portRotRad[7] = new MGROUP_ROTATE(uMeshIdx,radGrp27,2,
		_V(-1.95,1.25,-0.82),_V(0,0,1),(float)-140/180*PI);
	AddAnimationComponent(animPortRadiator,0,1,portRotRad[7],radiator26);

	//comp28
	static UINT radGrp28[1] = {99};
	portRotRad[8] = new MGROUP_ROTATE(uMeshIdx,radGrp28,1,
		_V(-1.46,0.17,-0.80),_V(0,0,1),(float)-65/180*PI);
	radiator28 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[8]);

	//comp29
	static UINT radGrp29[1] = {98};
	portRotRad[9] = new MGROUP_ROTATE(uMeshIdx,radGrp29,1,
		_V(-1.53,-0.54,-0.80),_V(0,0,1),(float)135/180*PI);
	radiator29 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[9],radiator28);

	//comp30
	static UINT radGrp30[1] = {97};
	portRotRad[10] = new MGROUP_ROTATE(uMeshIdx,radGrp30,1,
		_V(-1.60,1.25,-0.80),_V(0,0,1),(float)-140/180*PI);
	radiator30 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[10],radiator29);

	//comp31
	static UINT radGrp31[1] = {96};
	portRotRad[11] = new MGROUP_ROTATE(uMeshIdx,radGrp31,1,
		_V(-1.67,-0.54,-0.80),_V(0,0,1),(float)140/180*PI);
	radiator31 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[11],radiator30);

	//comp32
	static UINT radGrp32[1] = {95};
	portRotRad[12] = new MGROUP_ROTATE(uMeshIdx,radGrp32,1,
		_V(-1.74,1.25,-0.80),_V(0,0,1),(float)-140/180*PI);
	radiator32 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[12],radiator31);

	//comp33
	static UINT radGrp33[1] = {94};
	portRotRad[13] = new MGROUP_ROTATE(uMeshIdx,radGrp33,1,
		_V(-1.81,-0.54,-0.80),_V(0,0,1),(float)140/180*PI);
	radiator33 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[13],radiator32);

	//comp34
	static UINT radGrp34[1] = {93};
	portRotRad[14] = new MGROUP_ROTATE(uMeshIdx,radGrp34,1,
		_V(-1.88,1.25,-0.80),_V(0,0,1),(float)-140/180*PI);
	radiator34 = AddAnimationComponent(animPortRadiator,0,1,portRotRad[14],radiator33);

	//comp35 finally, YEAH!!!
	static UINT radGrp35[1] = {92};
	portRotRad[15] = new MGROUP_ROTATE(uMeshIdx,radGrp35,1,
		_V(-1.95,-0.54,-0.80),_V(0,0,1),(float)140/180*PI);
	AddAnimationComponent(animPortRadiator,0,1,portRotRad[15],radiator34);



	//STBD RADIATOR ANIMATIONS
	animStbdRadiator = CreateAnimation(0.0);

	//comp 38
	static UINT radGrp38[1] = {115};
	stbdRotRad[0] = new MGROUP_ROTATE(uMeshIdx,radGrp38,1,
		_V(1.39,0.52,3.73),_V(0,0,-1),(float)90/180*PI);
	ahStbdRad[0] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[0]);

	//comp 39
	static UINT radGrp39[1] = {114};
	stbdRotRad[1] = new MGROUP_ROTATE(uMeshIdx,radGrp39,1,
		_V(1.46,1.23,3.73),_V(0,0,-1),(float)-160/180*PI);
	ahStbdRad[1] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[1],ahStbdRad[0]);

	//comp 40
	static UINT radGrp40[1] = {113};
	stbdRotRad[2] = new MGROUP_ROTATE(uMeshIdx,radGrp40,1,
		_V(1.53,-0.57,3.73),_V(0,0,-1),(float)140/180*PI);
	ahStbdRad[2] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[2],ahStbdRad[1]);

	//comp41
	static UINT radGrp41[1] = {112};
	stbdRotRad[3] = new MGROUP_ROTATE(uMeshIdx,radGrp41,1,
		_V(1.60,1.23,3.73),_V(0,0,-1),(float)-140/180*PI);
	ahStbdRad[3] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[3],ahStbdRad[2]);

	//comp42
	static UINT radGrp42[1] = {111};
	stbdRotRad[4] = new MGROUP_ROTATE(uMeshIdx,radGrp42,1,
		_V(1.67,-0.57,3.73),_V(0,0,-1),(float)140/180*PI);
	ahStbdRad[4] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[4],ahStbdRad[3]);

	//comp43
	static UINT radGrp43[1] = {110};
	stbdRotRad[5] = new MGROUP_ROTATE(uMeshIdx,radGrp43,1,
		_V(1.74,1.23,3.73),_V(0,0,-1),(float)-140/180*PI);
	ahStbdRad[5] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[5],ahStbdRad[4]);

	//comp 44
	static UINT radGrp44[1] = {109};
	stbdRotRad[6] = new MGROUP_ROTATE(uMeshIdx,radGrp44,1,
		_V(1.81,-0.57,3.73),_V(0,0,-1),(float)140/180*PI);
	ahStbdRad[6] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[6],ahStbdRad[5]);

	//comp 45
	static UINT radGrp45[1] = {108};
	stbdRotRad[7] = new MGROUP_ROTATE(uMeshIdx,radGrp45,1,
		_V(1.88,1.23,3.73),_V(0,0,-1),(float)-140/180*PI);
	AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[7],ahStbdRad[6]);

	//comp 46
	static UINT radGrp46[1] = {123};
	stbdRotRad[8] = new MGROUP_ROTATE(uMeshIdx,radGrp46,1,
		_V(1.39,0.14,3.75),_V(0,0,-1),(float)-65/180*PI);
	ahStbdRad[7] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[8]);

	//comp 47
	static UINT radGrp47[1] = {122};
	stbdRotRad[9] = new MGROUP_ROTATE(uMeshIdx,radGrp47,1,
		_V(1.46,-0.57,3.75),_V(0,0,-1),(float)135/180*PI);
	ahStbdRad[8] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[9],ahStbdRad[7]);

	//comp 48
	static UINT radGrp48[1] = {121};
	stbdRotRad[10] = new MGROUP_ROTATE(uMeshIdx,radGrp48,1,
		_V(1.53,1.23,3.75),_V(0,0,-1),(float)-140/180*PI);
	ahStbdRad[9] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[10],ahStbdRad[8]);

	//comp 49
	static UINT radGrp49[1] = {120};
	stbdRotRad[11] = new MGROUP_ROTATE(uMeshIdx,radGrp49,1,
		_V(1.60,-0.57,3.75),_V(0,0,-1),(float)140/180*PI);
	ahStbdRad[10] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[11],ahStbdRad[9]);

	//comp 50
	static UINT radGrp50[1] = {119};
	stbdRotRad[12] = new MGROUP_ROTATE(uMeshIdx,radGrp50,1,
		_V(1.67,1.23,3.75),_V(0,0,-1),(float)-140/180*PI);
	ahStbdRad[11] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[12],ahStbdRad[10]);

	//comp 51
	static UINT radGrp51[1] = {118};
	stbdRotRad[13] = new MGROUP_ROTATE(uMeshIdx,radGrp51,1,
		_V(1.74,-0.57,3.73),_V(0,0,-1),(float)140/180*PI);
	ahStbdRad[12] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[13],ahStbdRad[11]);

	//comp 52
	static UINT radGrp52[1] = {117};
	stbdRotRad[14] = new MGROUP_ROTATE(uMeshIdx,radGrp52,1,
		_V(1.81,1.23,3.75),_V(0,0,-1),(float)-140/180*PI);
	ahStbdRad[13] = AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[14],ahStbdRad[12]);

	//comp 53
	static UINT radGrp53[1] = {116};
	stbdRotRad[15] = new MGROUP_ROTATE(uMeshIdx,radGrp53,1,
		_V(1.88,-0.57,3.75),_V(0,0,-1),(float)140/180*PI);
	AddAnimationComponent(animStbdRadiator,0,1,stbdRotRad[15],ahStbdRad[13]);

	//comp 54
	static UINT radGrp54[1] = {106};
	stbdScaleRad = new MGROUP_SCALE(uMeshIdx,radGrp54,1,_V(1.44,0.34,1.94),_V(0.1,1,1));
	AddAnimationComponent(animStbdRadiator,0,0.01,stbdScaleRad);


	//TOP RADIATOR
	animTopRadiator = CreateAnimation(0);

	//comp 55
	static UINT radGrp55[1] = {131};
	topRotRad[0] = new MGROUP_ROTATE(uMeshIdx,radGrp55,1,
		_V(-0.20,1.60,5.75),_V(0,0,-1),(float)90/180*PI);
	ahTopRad[0] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[0]);

	//comp 56
	static UINT radGrp56[1] = {130};
	topRotRad[1] = new MGROUP_ROTATE(uMeshIdx,radGrp56,1,
		_V(-0.91,1.67,5.75),_V(0,0,-1),(float)-160/180*PI);
	ahTopRad[1] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[1],ahTopRad[0]);

	//comp 57
	static UINT radGrp57[1] = {129};
	topRotRad[2] = new MGROUP_ROTATE(uMeshIdx,radGrp57,1,
		_V(0.89,1.74,5.75),_V(0,0,-1),(float)140/180*PI);
	ahTopRad[2] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[2],ahTopRad[1]);

	//comp58
	static UINT radGrp58[1] = {128};
	topRotRad[3] = new MGROUP_ROTATE(uMeshIdx,radGrp58,1,
		_V(-0.91,1.81,5.75),_V(0,0,-1),(float)-140/180*PI);
	ahTopRad[3] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[3],ahTopRad[2]);

	//comp 59
	static UINT radGrp59[1] = {127};
	topRotRad[4] = new MGROUP_ROTATE(uMeshIdx,radGrp59,1,
		_V(0.89,1.88,5.75),_V(0,0,-1),(float)140/180*PI);
	ahTopRad[4] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[4],ahTopRad[3]);

	//comp 60
	static UINT radGrp60[1] = {126};
	topRotRad[5] = new MGROUP_ROTATE(uMeshIdx,radGrp60,1,
		_V(-0.91,1.95,5.75),_V(0,0,-1),(float)-140/180*PI);
	ahTopRad[5] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[5],ahTopRad[4]);

	//comp 61
	static UINT radGrp61[1] = {125};
	topRotRad[6] = new MGROUP_ROTATE(uMeshIdx,radGrp61,1,
		_V(0.89,2.02,5.75),_V(0,0,-1),(float)140/180*PI);
	ahTopRad[6] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[6],ahTopRad[5]);

	//comp62
	static UINT radGrp62[1] = {124};
	topRotRad[7] = new MGROUP_ROTATE(uMeshIdx,radGrp62,1,
		_V(-0.91,2.09,5.75),_V(0,0,-1),(float)-140/180*PI);
	AddAnimationComponent(animTopRadiator,0,1,topRotRad[7],ahTopRad[6]);

	//comp 63
	static UINT radGrp63[1] = {139};
	topRotRad[8] = new MGROUP_ROTATE(uMeshIdx,radGrp63,1,
		_V(0.18,1.60,5.76),_V(0,0,-1),(float)-65/180*PI);
	ahTopRad[7] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[8]);

	//comp 64
	static UINT radGrp64[1] = {138};
	topRotRad[9] = new MGROUP_ROTATE(uMeshIdx,radGrp64,1,
		_V(0.89,1.67,5.76),_V(0,0,-1),(float)135/180*PI);
	ahTopRad[8] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[9],ahTopRad[7]);

	//comp 65
	static UINT radGrp65[1] = {137};
	topRotRad[10] = new MGROUP_ROTATE(uMeshIdx,radGrp65,1,
		_V(-0.91,1.74,5.76),_V(0,0,-1),(float)-140/180*PI);
	ahTopRad[9] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[10],ahTopRad[8]);

	//comp 66
	static UINT radGrp66[1] = {136};
	topRotRad[11] = new MGROUP_ROTATE(uMeshIdx,radGrp66,1,
		_V(0.89,1.81,5.76),_V(0,0,-1),(float)140/180*PI);
	ahTopRad[10] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[11],ahTopRad[9]);

	//comp 67
	static UINT radGrp67[1] = {135};
	topRotRad[12] = new MGROUP_ROTATE(uMeshIdx,radGrp67,1,
		_V(-0.91,1.88,5.76),_V(0,0,-1),(float)-140/180*PI);
	ahTopRad[11] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[12],ahTopRad[10]);

	//comp 68
	static UINT radGrp68[1] = {134};
	topRotRad[13] = new MGROUP_ROTATE(uMeshIdx,radGrp68,1,
		_V(0.89,1.95,5.76),_V(0,0,-1),(float)140/180*PI);
	ahTopRad[12] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[13],ahTopRad[11]);

	//comp 69
	static UINT radGrp69[1] = {133};
	topRotRad[14] = new MGROUP_ROTATE(uMeshIdx,radGrp69,1,
		_V(-0.91,2.02,5.76),_V(0,0,-1),(float)-140/180*PI);
	ahTopRad[13] = AddAnimationComponent(animTopRadiator,0,1,topRotRad[14],ahTopRad[12]);

	//comp 70
	static UINT radGrp70[1] = {132};
	topRotRad[15] = new MGROUP_ROTATE(uMeshIdx,radGrp70,1,
		_V(0.89,2.09,5.76),_V(0,0,-1),(float)140/180*PI);
	AddAnimationComponent(animTopRadiator,0,1,topRotRad[15],ahTopRad[13]);

	//comp 71
	static UINT radGrp71[1] = {107};
	topScaleRad = new MGROUP_SCALE(uMeshIdx,radGrp71,1,
		_V(0.0,1.55,3.91),_V(1,0.1,1));
	AddAnimationComponent(animTopRadiator,0,0.01,topScaleRad);

	


}

bool pv_p6::clbkDrawHUD(int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	double w = hps->W;
	DWORD csize = skp->GetCharSize();


	//CAPTIONS
	skp->SetTextColor(RGB(255,255,255));

	char cBottomAssembly[] = "Bottom assembly status:";
	skp->Text(20,200,cBottomAssembly,strlen(cBottomAssembly));

	char cTopAssembly[] = "Top assembly status:";
	skp->Text(20,220,cTopAssembly,strlen(cTopAssembly));

	char cTopArrayTrack[] = "Top array sun tracking:";
	skp->Text(20,240,cTopArrayTrack,strlen(cTopArrayTrack));

	char cBottomArrayTrack[] = "Bottom array sun tracking:";
	skp->Text(20,260,cBottomArrayTrack,strlen(cBottomArrayTrack));

	char cTopArrayAngle[] = "Top array rotation:";
	skp->Text(20,280,cTopArrayAngle,strlen(cTopArrayAngle));

	char cBottomArrayAngle[] = "Bottom array rotation:";
	skp->Text(20,300,cBottomArrayAngle,strlen(cBottomArrayAngle));

	char cTopArrayTgt[] = "Top array target angle:";
	skp->Text(20,320,cTopArrayTgt,strlen(cTopArrayTgt));

	char cBottomArrayTgt[] = "Bottom array target angle:";
	skp->Text(20,340,cBottomArrayTgt,strlen(cBottomArrayTgt));

	char cRotRate[] = "Array rotation rate:";
	skp->Text(20,360,cRotRate,strlen(cRotRate));

	char cRadiator[] = "Radiator status";
	skp->Text(20,380,cRadiator,strlen(cRadiator));

	char cStbdRadiator[] = "STBD Rad status";
	skp->Text(20,400,cStbdRadiator,strlen(cStbdRadiator));

	char cTopRadiator[] = "Top Rad status";
	skp->Text(20,420,cTopRadiator,strlen(cTopRadiator));



	char cbuf[256];


	//TOP ASSEMBLY STATES
	if(topAssembly.Closed())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Closed");
	}
	else if(topAssembly.Open())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Opened");
	}
	else if(topAssembly.Opening())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Opening");
	}
	else if(topAssembly.Closing())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Closing");
	}
	else if(topAssembly.Stopped())
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Stopped");
	}

	skp->Text(300,220,cbuf,strlen(cbuf));


	//BOTTOM ASSEMBLY STATES
	if(bottomAssembly.Closed())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Closed");
	}
	else if(bottomAssembly.Open())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Opened");
	}
	else if(bottomAssembly.Opening())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Opening");
	}
	else if(bottomAssembly.Closing())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Closing");
	}
	else if(bottomAssembly.Stopped())
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Stopped");
	}
	
	skp->Text(300,200,cbuf,strlen(cbuf));


	//TRACKING STATES
	if(topTracking)
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Enabled");
	}
	else
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Disabled");
	}
	skp->Text(300,240,cbuf,strlen(cbuf));

	if(bottomTracking)
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Enabled");
	}
	else
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Disabled");
	}
	skp->Text(300,260,cbuf,strlen(cbuf));

	//PORT ADIATOR STATES
	if(portRadiator.Closed())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Closed");
	}
	else if(portRadiator.Open())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Opened");
	}
	else if(portRadiator.Opening())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Opening");
	}
	else if(portRadiator.Closing())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Closing");
	}
	else if(portRadiator.Stopped())
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Stopped");
	}
	
	skp->Text(300,380,cbuf,strlen(cbuf));


	//STBD RADIATOR STATES

	if(stbdRadiator.Closed())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Closed");
	}
	else if(stbdRadiator.Open())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Opened");
	}
	else if(stbdRadiator.Opening())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Opening");
	}
	else if(stbdRadiator.Closing())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Closing");
	}
	else if(stbdRadiator.Stopped())
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Stopped");
	}
	
	skp->Text(300,400,cbuf,strlen(cbuf));

	//TOP RADIATOR STATES

	if(topRadiator.Closed())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Closed");
	}
	else if(topRadiator.Open())
	{
		skp->SetTextColor(RGB(0,255,0));
		strcpy(cbuf,"Opened");
	}
	else if(topRadiator.Opening())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Opening");
	}
	else if(topRadiator.Closing())
	{
		skp->SetTextColor(RGB(0,0,255));
		strcpy(cbuf,"Closing");
	}
	else if(topRadiator.Stopped())
	{
		skp->SetTextColor(RGB(255,0,0));
		strcpy(cbuf,"Stopped");
	}
	
	skp->Text(300,420,cbuf,strlen(cbuf));
	

	//ARRAY ANGLES
	skp->SetTextColor(RGB(255,255,0));
	sprintf(cbuf,"%.2lf",topArrayRot*360);
	skp->Text(300,280,cbuf,strlen(cbuf));

	sprintf(cbuf,"%.2lf",bottomArrayRot*360);
	skp->Text(300,300,cbuf,strlen(cbuf));


	//TARGET ANGLES
	skp->SetTextColor(RGB(255,125,10));
	sprintf(cbuf,"%.2lf",topArrayTgt*360);
	skp->Text(300,320,cbuf,strlen(cbuf));

	sprintf(cbuf,"%.2lf",bottomArrayTgt*360);
	skp->Text(300,340,cbuf,strlen(cbuf));


	//ROTATION RATE
	skp->SetTextColor(RGB(177,56,48));
	sprintf(cbuf,"%.4lf",arrayRotRate);
	skp->Text(300,360,cbuf,strlen(cbuf));


	return true;

}

void pv_p6::clbkSetClassCaps(FILEHANDLE cfg)
{
	uMeshIdx = AddMesh(hMesh);
	DefineAnimations();
}

void pv_p6::clbkPostStep(double simt, double simdt, double mjd)
{
	if(bottomAssembly.Moving())
	{
		bottomAssembly.Move(0.001*simdt);
		SetAnimation(animBottomAssembly,bottomAssembly.pos);
	}

	if(topAssembly.Moving())
	{
		topAssembly.Move(0.001*simdt);
		SetAnimation(animTopAssembly,topAssembly.pos);
	}

	if(portRadiator.Moving())
	{
		portRadiator.Move(0.001*simdt);
		SetAnimation(animPortRadiator,portRadiator.pos);
	}

	if(stbdRadiator.Moving())
	{
		stbdRadiator.Move(0.001*simdt);
		SetAnimation(animStbdRadiator,stbdRadiator.pos);
	}

	if(topRadiator.Moving())
	{
		topRadiator.Move(0.001*simdt);
		SetAnimation(animTopRadiator,topRadiator.pos);
	}

	if(topTracking)
	{
		if(topAssembly.Open())
		{

			topArrayTgt = CalculateSunTrackingAngle()/(PI*2);
			double d = topArrayTgt - topArrayRot;

			if(d>0)
			{
				if(fabs(d) >= arrayRotRate/360)
					topArrayRot += (arrayRotRate/360)*simdt;
				else 
					topArrayRot += fabs(d)*simdt;
			}
			else
			{
				if(fabs(d) >= arrayRotRate/360)
					topArrayRot -= (arrayRotRate/360)*simdt;
				else 
					topArrayRot -= fabs(d)*simdt;
			}

			if(topArrayRot > 1)
				topArrayRot = 0;

			if(topArrayRot < 0)
				topArrayRot = 1;


			if(abs(d)>0.01)
				SetAnimation(animTopArray,topArrayRot);
		}
	}

	else //navigate by custom target
	{
		if(topAssembly.Open())
		{

			double d = topArrayTgt - topArrayRot;

			if(d>0)
			{
				if(fabs(d) >= arrayRotRate/360)
					topArrayRot += (arrayRotRate/360)*simdt;
				else 
					topArrayRot += fabs(d)*simdt;
			}
			else
			{
				if(fabs(d) >= arrayRotRate/360)
					topArrayRot -= (arrayRotRate/360)*simdt;
				else 
					topArrayRot -= fabs(d)*simdt;
			}

			if(topArrayRot > 1)
				topArrayRot = 0;

			if(topArrayRot < 0)
				topArrayRot = 1;


			if(abs(d)>0.01)
				SetAnimation(animTopArray,topArrayRot);
		}
	} //topTracking



	if(bottomTracking)
	{
		if(bottomAssembly.Open())
		{

			bottomArrayTgt = CalculateSunTrackingAngle()/(PI*2);
			double d = bottomArrayTgt - bottomArrayRot;

			if(d>0)
			{
				if(fabs(d) >= arrayRotRate/360)
					bottomArrayRot += (arrayRotRate/360)*simdt;
				else 
					bottomArrayRot += fabs(d)*simdt;
			}
			else
			{
				if(fabs(d) >= arrayRotRate/360)
					bottomArrayRot -= (arrayRotRate/360)*simdt;
				else 
					bottomArrayRot -= fabs(d)*simdt;
			}

			if(bottomArrayRot > 1)
				bottomArrayRot = 0;

			if(bottomArrayRot < 0)
				bottomArrayRot = 1;

			if(abs(d)>0.01)
				SetAnimation(animBottomArray,bottomArrayRot);
		}
	}
	else
	{
		if(bottomAssembly.Open())
		{

			double d = bottomArrayTgt - bottomArrayRot;

			if(d>0)
			{
				if(fabs(d) >= arrayRotRate/360)
					bottomArrayRot += (arrayRotRate/360)*simdt;
				else 
					bottomArrayRot += fabs(d)*simdt;
			}
			else
			{
				if(fabs(d) >= arrayRotRate/360)
					bottomArrayRot -= (arrayRotRate/360)*simdt;
				else 
					bottomArrayRot -= fabs(d)*simdt;
			}

			if(bottomArrayRot > 1)
				bottomArrayRot = 0;

			if(bottomArrayRot < 0)
				bottomArrayRot = 1;

			if(abs(d)>0.01)
				SetAnimation(animBottomArray,bottomArrayRot);
		}
	}//bottomTracking
	
}

void pv_p6::OperateBottomAssembly(bool open)
{
	if(open)
	{
		if(bottomAssembly.action != AnimState::OPEN)
			bottomAssembly.action = AnimState::OPENING;
	}
	else
	{
		if(bottomArrayRot == 0 && !bottomTracking)
		{
			if(bottomAssembly.action != AnimState::CLOSED)
				bottomAssembly.action = AnimState::CLOSING;
		}
		else
		{
			sprintf(oapiDebugString(),"Rotate bottom array to 0 angle first");
		}
	}
}

void pv_p6::OperateTopAssembly(bool open)
{
	AnimState::Action action = topAssembly.action;

	if(open)
	{
		if(action != AnimState::OPEN)
			topAssembly.action = AnimState::OPENING;
	}
	else
	{
		if(topArrayRot == 0 && !topTracking)
		{
			if(action != AnimState::CLOSED)
				topAssembly.action = AnimState::CLOSING;
		}
		else
		{
			sprintf(oapiDebugString(),"Rotate top array to 0 angle first");
		}
	}
}

void pv_p6::OperateRadiator(bool open, int nbr)
{
	switch(nbr)
	{
	case 1:
		{
			AnimState::Action action = portRadiator.action;

			if(open)
			{
				if(action != AnimState::OPEN)
					portRadiator.action = AnimState::OPENING;
			}
			else
			{
				if(action != AnimState::CLOSED)
					portRadiator.action = AnimState::CLOSING;
			}

			break;
		}

	case 2:
		{
			AnimState::Action action = stbdRadiator.action;

			if(open)
			{
				if(action != AnimState::OPEN)
					stbdRadiator.action = AnimState::OPENING;
			}
			else
			{
				if(action != AnimState::CLOSED)
					stbdRadiator.action = AnimState::CLOSING;
			}

			break;
		}

	case 3:
		{
			AnimState::Action action = topRadiator.action;

			if(open)
			{
				if(action != AnimState::OPEN)
					topRadiator.action = AnimState::OPENING;
			}
			else
			{
				if(action != AnimState::CLOSED)
					topRadiator.action = AnimState::CLOSING;
			}

			break;
		}
	}
}

int pv_p6::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
	if(!down)
		return 0;

	if(KEYMOD_CONTROL(kstate))
	{
		switch(key)
		{
		case OAPI_KEY_1: {oapiOpenInputBox("Top array target angle[0-359]",clbkTopArrayTgt,"",20,this); return 1;}
		case OAPI_KEY_2: {oapiOpenInputBox("Bottom array target angle[0-359]",clbkBottomArrayTgt,"",20,this); return 1;}
		case OAPI_KEY_3: {oapiOpenInputBox("Array rotation rate[>=0]",clbkRotRate,"",20,this); return 1;}
		default: return 0;
		}
	}

	switch(key)
	{
	case OAPI_KEY_NUMPAD7: {OperateBottomAssembly(true); return 1;}
	case OAPI_KEY_NUMPAD4: {bottomAssembly.action = AnimState::STOPPED; return 1;}
	case OAPI_KEY_NUMPAD1: {OperateBottomAssembly(false); return 1;}
	case OAPI_KEY_NUMPAD8: {OperateTopAssembly(true); return 1;}
	case OAPI_KEY_NUMPAD5: {topAssembly.action = AnimState::STOPPED; return 1;}
	case OAPI_KEY_NUMPAD2: {OperateTopAssembly(false); return 1;}
	case OAPI_KEY_NUMPAD9: {OperateRadiator(true,1); return 1;}
	case OAPI_KEY_NUMPAD6: {portRadiator.action = AnimState::STOPPED; return 1;}
	case OAPI_KEY_NUMPAD3: {OperateRadiator(false,1); return 1;}
	case OAPI_KEY_3: {OperateRadiator(true,2); return 1;}
	case OAPI_KEY_4: {stbdRadiator.action = AnimState::STOPPED; return 1;}
	case OAPI_KEY_5: {OperateRadiator(false,2); return 1;}
	case OAPI_KEY_6: {OperateRadiator(true,3); return 1;}
	case OAPI_KEY_7: {topRadiator.action = AnimState::STOPPED; return 1;}
	case OAPI_KEY_8: {OperateRadiator(false,3); return 1;}
	case OAPI_KEY_1: {if(topTracking) topTracking = false; else topTracking = true; return 1;}
	case OAPI_KEY_2: {if(bottomTracking) bottomTracking = false; else bottomTracking = true; return 1;}
	default: return 0;
	}
}

bool pv_p6::clbkBottomArrayTgt(void *id, char *str, void *usrdata)
{
	pv_p6* pv = (pv_p6*)usrdata;

	double t;
	sscanf(str,"%lf",&t);

	if(t>=0 && t<=359)
	{
		pv->bottomArrayTgt = t/360;
		return true;
	}
	else return false;
}

bool pv_p6::clbkTopArrayTgt(void *id, char *str, void *usrdata)
{
	pv_p6* pv = (pv_p6*)usrdata;

	double t;
	sscanf(str,"%lf",&t);

	if(t>=0 && t<=359)
	{
		pv->topArrayTgt = t/360;
		return true;
	}
	else return false;
}

bool pv_p6::clbkRotRate(void *id, char *str, void *usrdata)
{
	pv_p6 *pv = (pv_p6*)usrdata;

	double t;
	sscanf(str,"%lf",&t);
	if(t>=0)
	{
		pv->arrayRotRate = t;
		return true;
	}
	else return false;
}

double pv_p6::CalculateSunTrackingAngle()
{
	VECTOR3 toSun, SunPos, ShipPos;
	oapiGetGlobalPos(oapiGetGbodyByName("Sun"), &SunPos);
	GetGlobalPos(ShipPos);

	MATRIX3 R;
	GetRotationMatrix(R);
	toSun = tmul(R, SunPos - ShipPos);

	double ang = atan2(toSun.x, toSun.z);
	if(ang < 0) ang += PI*2;

	return ang;
}

void pv_p6::clbkSaveState(FILEHANDLE scn)
{
	VESSEL3::clbkSaveState(scn);

	char cbuf[255];

	sprintf(cbuf,"%lf",arrayRotRate);
	oapiWriteScenario_string(scn,"ARRAY_RATE",cbuf);

	oapiWriteScenario_int(scn,"TOP_TRACKING",topTracking);
	oapiWriteScenario_int(scn,"BOTTOM_TRACKING",bottomTracking);

	sprintf(cbuf,"%lf",topArrayTgt);
	oapiWriteScenario_string(scn,"TOP_TGT",cbuf);

	sprintf(cbuf,"%lf",bottomArrayTgt);
	oapiWriteScenario_string(scn,"BOTTOM_TGT",cbuf);

	sprintf(cbuf,"%lf",topArrayRot);
	oapiWriteScenario_string(scn,"TOP_ROT",cbuf);

	sprintf(cbuf,"%lf",bottomArrayRot);
	oapiWriteScenario_string(scn,"BOTTOM_ROT",cbuf);

	sprintf(cbuf,"%i %lf",bottomAssembly.action,bottomAssembly.pos);
	oapiWriteScenario_string(scn,"ANIM_BOTTOM",cbuf);

	sprintf(cbuf,"%i %lf",topAssembly.action,topAssembly.pos);
	oapiWriteScenario_string(scn,"ANIM_TOP",cbuf);
	
	sprintf(cbuf,"%i %lf",portRadiator.action,portRadiator.pos);
	oapiWriteScenario_string(scn,"PORT_RAD",cbuf);

	sprintf(cbuf,"%i %lf",stbdRadiator.action,stbdRadiator.pos);
	oapiWriteScenario_string(scn,"STBD_RAD",cbuf);

	sprintf(cbuf,"%i %lf",topRadiator.action,topRadiator.pos);
	oapiWriteScenario_string(scn,"TOP_RAD",cbuf);
	
}

void pv_p6::clbkLoadStateEx(FILEHANDLE scn, void *status)
{
	char *line;

	while(oapiReadScenario_nextline(scn,line))
	{
		if(!_strnicmp(line,"ARRAY_RATE",10))
		{
			sscanf(line+10,"%lf",&arrayRotRate);
		}
		else if(!_strnicmp(line,"TOP_TRACKING",12))
		{
			sscanf(line+12,"%i",&topTracking);
		}
		else if(!_strnicmp(line,"BOTTOM_TRACKING",15))
		{
			sscanf(line+15,"%i",&bottomTracking);
		}
		else if(!_strnicmp(line,"TOP_TGT",7))
		{
			sscanf(line+7,"%lf",&topArrayTgt);
		}
		else if(!_strnicmp(line,"BOTTOM_TGT",10))
		{
			sscanf(line+10,"%lf",&bottomArrayTgt);
		}
		else if(!_strnicmp(line,"TOP_ROT",7))
		{
			sscanf(line+7,"%lf",&topArrayRot);
			SetAnimation(animTopArray,topArrayRot/360);
		}
		else if(!_strnicmp(line,"BOTTOM_ROT",10))
		{
			sscanf(line+10,"%lf",&bottomArrayRot);
			SetAnimation(animBottomArray,bottomArrayRot/360);
		}
		else if(!_strnicmp(line,"ANIM_BOTTOM",11))
		{
			int state; 
			double pos;
			sscanf(line+11,"%i %lf",&state,&pos);
			bottomAssembly.Set(GetAction(state),pos);
			SetAnimation(animBottomAssembly,pos);
		}
		else if(!_strnicmp(line,"ANIM_TOP",8))
		{
			int state;
			double pos;
			sscanf(line+8,"%i %lf",&state,&pos);
			topAssembly.Set(GetAction(state),pos);
			SetAnimation(animTopAssembly,pos);
		}
		else if(!_strnicmp(line,"PORT_RAD",8))
		{
			int state;
			double pos;
			sscanf(line+8,"%i %lf",&state,&pos);
			portRadiator.Set(GetAction(state),pos);
			SetAnimation(animPortRadiator,pos);
		}
		else if(!_strnicmp(line,"STBD_RAD",8))
		{
			int state;
			double pos;
			sscanf(line+8,"%i %lf",&state,&pos);
			stbdRadiator.Set(GetAction(state),pos);
			SetAnimation(animStbdRadiator,pos);
		}
		else if(!_strnicmp(line,"TOP_RAD",7))
		{
			int state;
			double pos;
			sscanf(line+7,"%i %lf",&state,&pos);
			topRadiator.Set(GetAction(state),pos);
			SetAnimation(animTopRadiator,pos);
		}
		else
			ParseScenarioLineEx(line,status);
	}

}

AnimState::Action pv_p6::GetAction(int a)
{
	switch(a)
	{
	case 1: return AnimState::CLOSED;
	case 2: return AnimState::OPEN;
	case 3: return AnimState::CLOSING;
	case 4: return AnimState::OPENING;
	case 0: return AnimState::STOPPED;
	}
}

DLLCLBK void InitModule (HINSTANCE hModule)
{
	g_hDLL = hModule;
	// perform global module initialisation here
}

DLLCLBK void ExitModule (HINSTANCE hModule)
{
	// perform module cleanup here
}

DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel) 
{ 
	return new pv_p6(hvessel, flightmodel); 
}

DLLCLBK void ovcExit (VESSEL *vessel) 
{ 
	if (vessel) 
		delete (pv_p6*)vessel; 
}

pv_p6::~pv_p6()
{

	for(int i=0; i<5; i++)
	{
		delete rotBottomAssembly[i];
		delete rotTopAssembly[i];
	}

	for(int i=0; i<3; i++)
	{
		delete scaleBottomAssembly[i];
		delete scaleTopAssembly[i];
	}

	delete transBottomAssembly;
	delete transTopAssembly;
	delete rotBottomArray;
	delete rotTopArray;
	
	for(int i=0; i<16; i++)
	{
		delete portRotRad[i];
		delete stbdRotRad[i];
		delete topRotRad[i];
	}


}
